@@ -1,53 +1,127 @@
+=head1 NAME
+
+Clustericious::Config - configuration files for Clustericious nodes.
+
+=head1 SYNOPSIS
+
+ $ cat > ~/etc/MyApp.conf
+ ---
+ % extends_config 'global';
+ % extends_config 'hypnotoad', url => 'http://localhost:9999', app => 'MyApp';
+
+ url : http://localhost:9999
+ start_mode : hypnotoad
+ hypnotoad :
+    - heartbeat_timeout : 500
+
+ $ cat > ~/global.conf
+ ---
+ somevar : somevalue
+
+ $ cat > ~/hypnotoad.conf
+ listen :
+    - <%= $url %>
+ pid_file : <%= $ENV{HOME} %>/<%= $app %>/hypnotoad.pid
+ env :
+    MOJO_HOME : <%= $ENV{HOME} %>/<%= $app %>
+
+Then later in a program somewhere :
+
+ my $c = Clustericious::Config->new("MyApp");
+ my $c = Clustericious::Config->new( \$config_string );
+ my $c = Clustericious::Config->new( \%config_data_structure );
+
+ print $c->url;
+ print $c->{url};
+
+ print $c->hypnotoad->listen;
+ print $c->hypnotoad->{listen};
+ my %hash = $c->hypnotoad;
+ my @ary  = $c->hypnotoad;
+
+ # Supply a default value for a missing configuration parameter :
+ $c->url(default => "http://localhost:9999");
+ print $c->this_param_is_missing(default => "something_else");
+
+ # Dump out the entire config as yaml
+ print $c->dump_as_yaml;
+
+=head1 DESCRIPTION
+
+Clustericious::Config reads configuration files which are Mojo::Template's
+of JSON or YAML files.  There should generally be an entry for
+'url', which may be used by either a client or a server depending on
+how this node in the cluster is being used.
+
+After rendering the template and parsing the JSON, the resulting
+object may be called using method calls or treated as hashes.
+
+Config files are looked for in the following places (in order, where
+"MyApp" is the name of the app) :
+
+    $CLUSTERICIOUS_CONF_DIR/MyApp.conf
+    $HOME/etc/MyApp.conf
+    /util/etc/MyApp.conf
+    /etc/MyApp.conf
+
+The helper "extends_config" may be used to read default settings
+from another config file.  The first argument to extends_config is the
+basename of the config file.  Additional named arguments may be passed
+to that config file and used as variables within that file.  After
+reading another file, the hashes are merged (i.e. with Hash::Merge);
+so values anywhere inside the datastructure may be overridden.
+
+YAML config files must begin with "---", otherwise they are interpreted
+as JSON.
+
+Clustericious::Config provides a "get_password" function which will prompt
+for a password if it is needed.  It can be used like this :
+
+ password : <%= get_password =%>
+
+This will prompt the user the first time it is encountered.
+
+=head1 METHODS
+
+=over
+
+=cut
+
 package Clustericious::Config;
 
+use Clustericious::Config::Password;
+
 use strict;
 use warnings;
-use v5.10;
 
-# ABSTRACT: configuration files for Clustericious nodes.
-our $VERSION = '0.24_05'; # VERSION
+our $VERSION = '0.10';
 
-
-use Clustericious::Config::Password;
-use List::Util;
+use List::Util qw/first/;
 use JSON::XS;
-use YAML::XS ();
+use YAML::XS qw/Load Dump/;
 use Mojo::Template;
 use Log::Log4perl qw/:easy/;
-use Storable;
-use Clustericious::Config::Helpers ();
+use Storable qw/dclone/;
+use Clustericious::Config::Plugin;
 use Data::Dumper;
-use Cwd ();
+use Cwd qw/getcwd abs_path/;
 use Module::Build;
-use File::HomeDir ();
 
 our %Singletons;
 
 sub _is_subdir {
     my ($child,$parent) = @_;
-    my $p = Cwd::abs_path($parent);
-    my $c = Cwd::abs_path($child);
+    my $p = abs_path($parent);
+    my $c = abs_path($child);
     return ($c =~ m[^\Q$p\E]) ? 1 : 0;
 }
 
-my $is_test = 0;
-sub _testing {
-  my($class, $new) = @_;
-  $is_test = $new if defined $new;
-  $is_test;
-}
+=item new
 
-our $class_suffix = {};
-sub _uncache {
-    my($class, $name) = @_;
-    delete $Clustericious::Config::Singletons{$name};
-    $class_suffix->{$name} //= 1;
-    $class_suffix->{$name}++;
-}
-
-sub pre_rendered { }
-sub rendered { }
+Create a new Clustericious::Config object.  See the SYPNOSIS for
+possible invocations.
 
+=cut
 
 sub new {
     my $class = shift;
@@ -56,11 +130,8 @@ sub new {
     ($arg = caller) =~ s/:.*$// unless $arg; # Determine from caller's class
     return $Singletons{$arg} if exists($Singletons{$arg});
 
-    
     my $we_are_testing_this_module = 0;
-    if(__PACKAGE__->_testing) {
-        $we_are_testing_this_module = 0;
-    } elsif ($ENV{HARNESS_ACTIVE} and -d '_build' && -e '_build/build_params' && Module::Build->can("current")) {
+    if ($ENV{HARNESS_ACTIVE} and Module::Build->can("current")) {
         my $mb = Module::Build->current;
         $we_are_testing_this_module = $mb && $mb->module_name eq $arg;
     }
@@ -68,34 +139,25 @@ sub new {
     my $conf_data;
 
     my $json = JSON::XS->new;
-    
-    state $package_counter = 0;
-    my $namespace = "Clustericious::Config::TemplatePackage::Package$package_counter";
-    eval qq{ package $namespace; use Clustericious::Config::Helpers; };
-    die $@ if $@;
-    $package_counter++;
-    
-    my $mt = Mojo::Template->new(namespace => $namespace)->auto_escape(0);
+    my $mt = Mojo::Template->new(namespace => 'Clustericious::Config::Plugin')->auto_escape(0);
     $mt->prepend( join "\n", map " my \$$_ = q{$t_args{$_}};", sort keys %t_args );
 
     my $filename;
     if (ref $arg eq 'SCALAR') {
-        $class->pre_rendered( $$arg );
         my $rendered = $mt->render($$arg);
-        $class->rendered( SCALAR => $rendered );
         die $rendered if ( (ref($rendered)) =~ /Exception/ );
         my $type = $rendered =~ /^---/ ? 'yaml' : 'json';
         $conf_data = $type eq 'yaml' ?
-           eval { YAML::XS::Load( $rendered ); }
+           eval { Load( $rendered ); }
          : eval { $json->decode( $rendered ); };
         LOGDIE "Could not parse $type \n-------\n$rendered\n---------\n$@\n" if $@;
     } elsif (ref $arg eq 'HASH') {
-        $conf_data = Storable::dclone $arg;
+        $conf_data = dclone $arg;
     } elsif (
           $we_are_testing_this_module
           && !(
               $ENV{CLUSTERICIOUS_CONF_DIR}
-              && _is_subdir( $ENV{CLUSTERICIOUS_CONF_DIR}, Cwd::getcwd() )
+              && _is_subdir( $ENV{CLUSTERICIOUS_CONF_DIR}, getcwd() )
           )) {
           $conf_data = {};
     } else {
@@ -103,16 +165,13 @@ sub new {
 
         @conf_dirs = $ENV{CLUSTERICIOUS_CONF_DIR} if defined( $ENV{CLUSTERICIOUS_CONF_DIR} );
 
-        push @conf_dirs, ( File::HomeDir->my_home . "/etc", "/util/etc", "/etc" ) unless $we_are_testing_this_module || __PACKAGE__->_testing;
+        push @conf_dirs, ( "$ENV{HOME}/etc", "/util/etc", "/etc" ) unless $we_are_testing_this_module;
         my $conf_file = "$arg.conf";
-        $conf_file =~ s/::/-/g;
-        my ($dir) = List::Util::first { -e "$_/$conf_file" } @conf_dirs;
+        my ($dir) = first { -e "$_/$conf_file" } @conf_dirs;
         if ($dir) {
             TRACE "reading from config file $dir/$conf_file";
             $filename = "$dir/$conf_file";
-            $class->pre_rendered( $filename );
-            my $rendered = $mt->render_file($filename);
-            $class->rendered( $filename => $rendered );
+            my $rendered = $mt->render_file("$dir/$conf_file");
             die $rendered if ( (ref $rendered) =~ /Exception/ );
             my $type = $rendered =~ /^---/ ? 'yaml' : 'json';
             if ($ENV{CL_CONF_TRACE}) {
@@ -121,7 +180,7 @@ sub new {
             }
             $conf_data =
               $type eq 'yaml'
-              ? eval { YAML::XS::Load($rendered) }
+              ? eval { Load($rendered) }
               : eval { $json->decode($rendered) };
             LOGDIE "Could not parse $type\n-------\n$rendered\n---------\n$@\n" if $@;
         } else {
@@ -130,7 +189,7 @@ sub new {
         }
     }
     $conf_data ||= {};
-    Clustericious::Config::Helpers->_do_merges($conf_data);
+    Clustericious::Config::Plugin->do_merges($conf_data);
     _add_heuristics($filename,$conf_data);
     # Use derived classes so that AUTOLOADING keeps namespaces separate
     # for various apps.
@@ -139,8 +198,7 @@ sub new {
             $arg = "$arg";
             $arg =~ tr/a-zA-Z0-9//cd;
         }
-        $class = join '::', $class, 'App', $arg;
-        $class .= $class_suffix->{$arg} if $class_suffix->{$arg};
+        $class = join '::', $class, $arg;
         my $dome = '@'."$class"."::ISA = ('".__PACKAGE__. "')";
         eval $dome;
         die "error setting ISA : $@" if $@;
@@ -160,10 +218,9 @@ sub _add_heuristics {
 
 }
 
-
 sub dump_as_yaml {
     my $c = shift;
-    return YAML::XS::Dump($c);
+    return Dump($c);
 }
 
 sub _stringify {
@@ -212,6 +269,13 @@ sub AUTOLOAD {
     $self->$called;
 }
 
+=item set_singleton
+
+Clustericicious::Config->set_singleton(App => $object);
+
+Cache a config object to be returned by the constructor.
+
+=cut
 
 sub set_singleton {
     my $class = shift;
@@ -221,143 +285,7 @@ sub set_singleton {
     $Singletons{$app} = $obj;
 }
 
-
-1;
-
-
-__END__
-=pod
-
-=head1 NAME
-
-Clustericious::Config - configuration files for Clustericious nodes.
-
-=head1 VERSION
-
-version 0.24_05
-
-=head1 SYNOPSIS
-
-In your ~/etc/MyApp.conf file:
-
- ---
- % extends_config 'global';
- % extends_config 'hypnotoad', url => 'http://localhost:9999', app => 'MyApp';
-
- url : http://localhost:9999
- start_mode : hypnotoad
- hypnotoad :
-    - heartbeat_timeout : 500
- 
- arbitrary_key: value
-
-In your ~/etc/globa.conf file:
-
- ---
- somevar : somevalue
-
-In your ~/etc/hypnotoad.conf:
-
- listen :
-    - <%= $url %>
- # home uses File::HomeDir to find the calling users'
- # home directory
- pid_file : <%= home %>/<%= $app %>/hypnotoad.pid
- env :
-    MOJO_HOME : <%= home %>/<%= $app %>
-
-From a L<Clustericious::App>:
-
- package MyApp;
- 
- use Mojo::Base qw( Clustericious::App );
- 
- package MyApp::Routes;
- 
- use Clustericious::RouteBuilder;
- 
- get '/' => sub {
-   my $c = shift;
-   my $config = $c; # $config isa Clustericious::Config
-   
-   # returns the value if it is defined, foo otherwise
-   my $value = $config->arbitrary_key(default => 'foo');
- };
-
-From a script:
-
- use Clustericious::Config;
- 
- my $c = Clustericious::Config->new("MyApp");
- my $c = Clustericious::Config->new( \$config_string );
- my $c = Clustericious::Config->new( \%config_data_structure );
-
- print $c->url;
- print $c->{url};
-
- print $c->hypnotoad->listen;
- print $c->hypnotoad->{listen};
- my %hash = $c->hypnotoad;
- my @ary  = $c->hypnotoad;
-
- # Supply a default value for a missing configuration parameter :
- $c->url(default => "http://localhost:9999");
- print $c->this_param_is_missing(default => "something_else");
-
- # Dump out the entire config as yaml
- print $c->dump_as_yaml;
-
-=head1 DESCRIPTION
-
-Clustericious::Config reads configuration files which are Mojo::Template's
-of JSON or YAML files.  There should generally be an entry for
-'url', which may be used by either a client or a server depending on
-how this node in the cluster is being used.
-
-After rendering the template and parsing the JSON, the resulting
-object may be called using method calls or treated as hashes.
-
-Config files are looked for in the following places (in order, where
-"MyApp" is the name of the app) :
-
-    $CLUSTERICIOUS_CONF_DIR/MyApp.conf
-    $HOME/etc/MyApp.conf
-    /util/etc/MyApp.conf
-    /etc/MyApp.conf
-
-The helper "extends_config" may be used to read default settings
-from another config file.  The first argument to extends_config is the
-basename of the config file.  Additional named arguments may be passed
-to that config file and used as variables within that file.  After
-reading another file, the hashes are merged (i.e. with Hash::Merge);
-so values anywhere inside the data structure may be overridden.
-
-YAML config files must begin with "---", otherwise they are interpreted
-as JSON.
-
-This module provides a number of helpers
-which can be used to get system details (such as the home directory of
-the calling user or to prompt for passwords).  See L<Clustericious::Config::Helpers>
-for details.
-
-=head1 CONSTRUCTOR
-
-=head2 new
-
-Create a new Clustericious::Config object.  See the SYNOPSIS for
-possible invocations.
-
-=head1 METHODS
-
-=head2 $config-E<gt>dump_as_yaml
-
-Returns a string with the configuration encoded as YAML.
-
-=head2 Clustericious::Config->set_singleton
-
-Cache a config object to be returned by the constructor.  Usage:
-
- Clustericicious::Config->set_singleton(App => $object);
+=back
 
 =head1 ENVIRONMENT
 
@@ -365,20 +293,11 @@ If the environment variable HARNESS_ACTIVE is set,
 and the current module::build object tells us that
 the calling module is being tested, then an empty
 configuration is used.  In this situation, however,
-if the CLUSTERICIOUS_CONF_DIR environment variable
-is set and if it is a subdirectory of the current 
-directory, then it will be used.  This allows unit 
-tests to provide configuration directories, but 
-avoids using configurations that are outside of 
-the build tree during unit testing.
-
-=head1 CAVEATS
-
-Some filesystems do not support filenames with a colon
-(:) character in them, so for apps with a double colon
-in them (for example L<Clustericious::HelloWorld>),
-a single dash character will be substituted for the name
-(for example C<Clustericious-HelloWorld.conf>).
+if $ENV{CLUSTERICIOUS_CONF_DIR} is set and if it
+is a subdirectory of the current directory, then
+it will be used.  This allows unit tests to provide
+configuration directories, but avoids using configurations
+that are outside of the build tree during unit testing.
 
 =head1 NOTES
 
@@ -386,20 +305,9 @@ This is a beta release. The API may change without notice.
 
 =head1 SEE ALSO
 
-L<Mojo::Template>, L<Hash::Merge>, L<Clustericious>, L<Clustericious::Client>, L<Clustericious::Config::Helpers>
-
-=head1 AUTHOR
-
-original author: Brian Duggan
-
-current maintainer: Graham Ollis <plicease@cpan.org>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2013 by NASA GSFC.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
+L<Mojo::Template>, L<Hash::Merge>, L<Clustericious>, L<Clustericious::Client>
 
 =cut
 
+1;
+
